#' Extrapolacion Temporal de series medidas en sitio
#'
#' @param torre_sitio Objeto \code{torre} que contiene las series medidas en sitio
#' @param torresec Objeto \code{torre} que contiene las series de la fuente secundaria
#' @param torresecomun Objeto \code{torre} que contiene las series de la fuente secundaria para el periodo comun
#' @param alturasecomun Altura de las series de la fuente secundaria
#' @param alpha Cortante del viento en el sitio de frecuencia diezminutal
#' @param alphameses Cortante del viento caracterizado con días típicos mensuales.
#' @param seriesel Serie de Dirección Seleccionada
#' @param indiceTorre Índice de Torre para el cual se ejecuta MCP
#'
#' @details Se ejecuta el MCP entre la serie de datos medidos en sitio y las fuentes secundarias.
#' Si la fuente secundaria tiene la misma altura que un nivel de las mediciones en sitio, no se extrapolan los datos secundarios.
#' Si la fuente secundaria está a una altura superior a las alturas medidas en sitio, se extrapola la velocidad superior medida en sitio a la altura secundaria.
#' Si la fuente secundaria tiene una altura diferente entre las alturas medidas en sitio o inferior, se extrapola al nivel más cercano y se ejecuta el MCP.
#' 
#' @return La altura de referencia de la reconstrucción temporal.
#' Las series reconstruídas son generadas como archivos .csv en la dirección del paquete. De esta forma, son trabajados por el aplicativo principal
#' 
#' @export
#'
#' @examples ##Ejemplo

MCP <- function(torre_sitio, torresec, torresecomun, alturasecomun, alpha, alphameses, seriesel, indiceTorre) {
  altigual <- FALSE
  for (i in 1:length(torre_sitio$series)) {
    if (torre_sitio$series[[i]]$altura==alturasecomun) {
      altigual <- TRUE
      VelIgual <- torre_sitio$series[[i]]$data$v.prom
      alturaRef <- torre_sitio$series[[i]]$altura
    }
  }
  altMax <- torre_sitio$series[[length(torre_sitio$series)]]$altura

  if (altigual==TRUE) { #Si la altura de la serie secundaria es igual a alguna de la medidas en sitio
    #Se convierten las series de velocidades y direcciones a frecuencia horaria
    orig <- ts(VelIgual, start=c(as.numeric(format(as.Date(torre_sitio$marcatemporal[1]),'%Y')),as.numeric(format(as.Date(torre_sitio$marcatemporal[1]),'%m'))),
               frequency=52560)
    VelHor <- as.numeric(aggregate(orig, nfrequency=8760)/6)

    orig <- ts(seriesel, start=c(as.numeric(format(as.Date(torre_sitio$marcatemporal[1]),'%Y')), as.numeric(format(as.Date(torre_sitio$marcatemporal[1]),'%m'))),
               frequency=52560)
    DirHor <- as.numeric(aggregate(orig, nfrequency=8760)/6)

    fecha_horaria <- seq(from=as.POSIXct(torre_sitio$marcatemporal[1]),to=as.POSIXct(torre_sitio$marcatemporal[length(torre_sitio$marcatemporal)]), by="hour")
    if (!length(fecha_horaria)==length(VelHor)) {
      fecha_horaria <- fecha_horaria[-length(fecha_horaria)]
    } else {}
    titulos_sitio=c("Fechas","Velocidad","Direccion")
    TABLE_sitio=data.frame(fecha_horaria,VelHor,DirHor)
    names(TABLE_sitio)=titulos_sitio
    write.csv(x = TABLE_sitio,file = paste(getwd(), "/Serie_Sitio_Horaria-Torre", indiceTorre, ".csv", sep = ""))

    titulos_sec=c("Fechas","Velocidad","Direccion")
    TABLE_sec=data.frame(as.POSIXct(torresec$marcatemporal),torresec$series$serie1$data$v.prom,
                         torresec$series$serie1$data$dir.prom)
    names(TABLE_sec)=titulos_sec
    write.csv(x = TABLE_sec,file = paste(getwd(), "/Serie_Fuente_Secundaria-Torre", indiceTorre, ".csv", sep = ""))

    titulos_secomun=c("Fechas","Velocidad","Direccion")
    TABLE_secomun=data.frame(as.POSIXct(torresecomun$marcatemporal),torresecomun$series$serie1$data$v.prom,
                             torresecomun$series$serie1$data$dir.prom)
    names(TABLE_secomun)=titulos_secomun
    write.csv(x = TABLE_secomun,file = paste(getwd(), "/Serie_Fuente_Secundaria_PeriodoComun-Torre", indiceTorre, ".csv", sep = ""))
    write.csv(x = indiceTorre, file = paste(getwd(), "/indiceTorre.csv", sep = ""))

    cat("Se ejecuta la extrapolación temporal realizada con MCP
        ")
    source(paste(getwd(), "/www/R/PROTOCOLO_R_HISTORICA.R", sep = ""), local = T)

  } else if (alturasecomun>altMax) { #Si la altura de la serie secundaria es superior a todas las alturas de medición en sitio
    alturaRef <- alturasecomun
    Vel_sitioext <- torre_sitio$series[[length(torre_sitio$series)]]$data$v.prom*
      (alturaRef/torresecomun$series$serie1$altura)^alpha
    #Se convierten las series de velocidades y direcciones a frecuencia horaria
    orig <- ts(Vel_sitioext, start=c(as.numeric(format(as.Date(torre_sitio$marcatemporal[1]),'%Y')),as.numeric(format(as.Date(torre_sitio$marcatemporal[1]),'%m'))),
               frequency=52560)
    VelHor <- as.numeric(aggregate(orig, nfrequency=8760)/6)

    orig <- ts(seriesel, start=c(as.numeric(format(as.Date(torre_sitio$marcatemporal[1]),'%Y')), as.numeric(format(as.Date(torre_sitio$marcatemporal[1]),'%m'))),
               frequency=52560)
    DirHor <- as.numeric(aggregate(orig, nfrequency=8760)/6)

    fecha_horaria <- seq(from=as.POSIXct(torre_sitio$marcatemporal[1]),to=as.POSIXct(torre_sitio$marcatemporal[length(torre_sitio$marcatemporal)]), by="hour")
    if (!length(fecha_horaria)==length(VelHor)) {
      fecha_horaria <- fecha_horaria[-length(fecha_horaria)]
    } else {}
    titulos_sitio=c("Fechas","Velocidad","Direccion")
    TABLE_sitio=data.frame(fecha_horaria,VelHor,DirHor)
    names(TABLE_sitio)=titulos_sitio
    write.csv(x = TABLE_sitio,file = paste(getwd(), "/Serie_Sitio_Horaria-Torre", indiceTorre, ".csv", sep = ""))

    titulos_sec=c("Fechas","Velocidad","Direccion")
    TABLE_sec=data.frame(as.POSIXct(torresec$marcatemporal),torresec$series$serie1$data$v.prom,
                         torresec$series$serie1$data$dir.prom)
    names(TABLE_sec)=titulos_sec
    write.csv(x = TABLE_sec,file = paste(getwd(), "/Serie_Fuente_Secundaria-Torre", indiceTorre, ".csv", sep = ""))

    titulos_secomun=c("Fechas","Velocidad","Direccion")
    TABLE_secomun=data.frame(as.POSIXct(torresecomun$marcatemporal),torresecomun$series$serie1$data$v.prom,
                             torresecomun$series$serie1$data$dir.prom)
    names(TABLE_secomun)=titulos_secomun
    write.csv(x = TABLE_secomun,file = paste(getwd(), "/Serie_Fuente_Secundaria_PeriodoComun-Torre", indiceTorre, ".csv", sep = ""))
    write.csv(x = indiceTorre, file = paste(getwd(), "/indiceTorre.csv", sep = ""))

    cat("Se ejecuta la extrapolación temporal realizada con MCP
        ")
    source(paste(getwd(), "/www/R/PROTOCOLO_R_HISTORICA.R", sep = ""), local = T)

  } else { #Si la altura de la serie secundaria no coincide con las anteriores condiciones
    dif <- rep(0,length(torre_sitio$series))
    for (i in 1:length(torre_sitio$series)) {
      dif[i] <- abs(torre_sitio$series[[i]]$altura-alturasecomun[[1]])
    }
    indice <- which.min(dif)
    alturaRef <- torre_sitio$series[[indice]]$altura
    #Se convierten las series de velocidades y direcciones a frecuencia horaria
    orig <- ts(torre_sitio$series[[indice]]$data$v.prom,
               start=c(as.numeric(format(as.Date(torre_sitio$marcatemporal[1]),'%Y')),as.numeric(format(as.Date(torre_sitio$marcatemporal[1]),'%m'))),
               frequency=52560)
    VelHor <- as.numeric(aggregate(orig, nfrequency=8760)/6)

    orig <- ts(seriesel, start=c(as.numeric(format(as.Date(torre_sitio$marcatemporal[1]),'%Y')),as.numeric(format(as.Date(torre_sitio$marcatemporal[1]),'%m'))),
               frequency=52560)
    DirHor <- as.numeric(aggregate(orig, nfrequency=8760)/6)

    fecha_horaria <- seq(from=as.POSIXct(torre_sitio$marcatemporal[1]),to=as.POSIXct(torre_sitio$marcatemporal[length(torre_sitio$marcatemporal)]), by="hour")
    if (!length(fecha_horaria)==length(VelHor)) {
      fecha_horaria <- fecha_horaria[-length(fecha_horaria)]
    } else {}
    titulos_sitio=c("Fechas","Velocidad","Direccion")
    TABLE_sitio=data.frame(fecha_horaria,VelHor,DirHor)
    names(TABLE_sitio)=titulos_sitio
    write.csv(x = TABLE_sitio,file = paste(getwd(), "/Serie_Sitio_Horaria-Torre", indiceTorre, ".csv", sep = ""))

    Vel_secomun <- torresecomun$series$serie1$data$v.prom

    for (i in 1:length(torresecomun$series$serie1$data$v.prom)) {
      month <- format(torresecomun$marcatemporal[i], "%m")
      hora <- as.numeric(format(torresecomun$marcatemporal[i], "%H"))+1
      Vel_secomun[i] <- torresecomun$series$serie1$data$v.prom[i]*
        (alturaRef/torresecomun$series$serie1$altura)^alphameses[[month]][,2][hora]
    }
    titulos_secomun=c("Fechas","Velocidad","Direccion")
    TABLE_secomun=data.frame(as.POSIXct(torresecomun$marcatemporal),Vel_secomun,
                             torresecomun$series$serie1$data$dir.prom)
    names(TABLE_secomun)=titulos_secomun
    write.csv(x = TABLE_secomun,file = paste(getwd(), "/Serie_Fuente_Secundaria_PeriodoComun-Torre", indiceTorre, ".csv", sep = ""))

    Vel_sec <- torresec$series$serie1$data$v.prom

    for (i in 1:length(torresec$series$serie1$data$v.prom)) {
      month <- format(torresec$marcatemporal[i], "%m")
      hora <- as.numeric(format(torresec$marcatemporal[i], "%H"))+1
      Vel_sec[i] <- torresec$series$serie1$data$v.prom[i]*
        (alturaRef/torresec$series$serie1$altura)^alphameses[[month]][,2][hora]
    }
    titulos_sec=c("Fechas","Velocidad","Direccion")
    TABLE_sec=data.frame(as.POSIXct(torresec$marcatemporal),Vel_sec,
                         torresec$series$serie1$data$dir.prom)
    names(TABLE_sec)=titulos_sec
    write.csv(x = TABLE_sec,file = paste(getwd(), "/Serie_Fuente_Secundaria-Torre", indiceTorre, ".csv", sep = ""))
    write.csv(x = indiceTorre, file = paste(getwd(), "/indiceTorre.csv", sep = ""))

    cat("Se ejecuta la extrapolación temporal realizada con MCP
        ")
    source(paste(getwd(), "/www/R/PROTOCOLO_R_HISTORICA.R", sep = ""), local = T)
  }
  return(alturaRef)
}
