#' Importa una curva de potencia
#'
#' @description Importa una curva de potencia hecha por el usuario,
#' @param curvpot a
#' @param ... b
#'
#' @return c
#'
#' @export
#'
#' @examples d
#'
curvaPotencia <- function (curvpot, ...)
{
  if (is.list(curvpot) || is.data.frame(curvpot))
    r <- curvpot.default(curvpot, ...)
  else if (is.character(curvpot))
    r <- curvpot.leer(curvpot, ...)
  else stop(substitute(curvpot), "debe ser un vector numérico de velocidades de viento, el nombre de este, o la ruta a un archivo 'wgt' o 'pow'
            que contiene la curva de potencia")
  return(r)
}

curvpot.default <-
  function(curvpot, rho=1.225, rated.p, desc, ...) {
    ###	Creando un objeto curva de Potencia

    v <- p <- cp <- ct <- NULL
    if(any(names(curvpot)!=c(v, p, cp, ct))) stop("Variable(s) not recognized - power curve data may contain 'v', 'p', 'cp' and/or 'ct'")
    if(length(names(curvpot))>0) {
      if(any(names(curvpot)=="")) stop("Please name all power curve variables")
      if(any(names(curvpot)=="v")) v <- curvpot$v
      else stop("Wind speed 'v' is mandatory")
      if(any(names(curvpot)=="p")) p <- curvpot$p
      else stop("Power 'p' is mandatory")
      if(any(names(curvpot)=="cp")) cp <- curvpot$cp
      if(any(names(curvpot)=="ct")) ct <- curvpot$ct
    } else {
      if(is.list(curvpot)) {
        if(length(curvpot)>0) v <- curvpot[[1]]
        else stop("Wind speed 'v' is mandatory")
        if(length(curvpot)>1) p <- curvpot[[2]]
        else stop("Power 'p' is mandatory")
        if(length(curvpot)>2) cp <- curvpot[[3]]
        if(length(curvpot)>3) ct <- curvpot[[4]]
      } else if(is.data.frame(curvpot)) {
        if(length(curvpot)>0) v <- curvpot[,1]
        else stop("Wind speed 'v' is mandatory")
        if(length(curvpot)>1) p <- curvpot[,2]
        else stop("Power 'p' is mandatory")
        if(length(curvpot)>2) cp <- curvpot[,3]
        if(length(curvpot)>3) ct <- curvpot[,4]
      }
    }
    if(!is.vector(v)) stop("'v' requires numeric vector")
    if(!is.vector(p)) stop("'p' requires numeric vector")
    if(length(v)!=length(p)) stop("Different vector length of 'v' and 'p'")
    if(!is.null(cp)) if(!is.vector(cp)) stop("'cp' requires numeric vector")
    if(!is.null(cp)) if(length(v)!=length(cp)) stop("Different vector length of 'v' and 'cp'")
    if(!is.null(ct)) if(!is.vector(ct)) stop("'ct' requires numeric vector")
    if(!is.null(ct)) if(length(v)!=length(ct)) stop("Different vector length of 'v' and 'ct'")
    if(missing(rho)) rho <- 1.225
    if(missing(rated.p)) rated.p <- max(p, na.rm=TRUE)
    if(missing(desc)) desc <- NULL

    curvpot <- data.frame(cbind(v, p, cp, ct))
    names <- c("v", "P")
    attr(curvpot, "units") <- c("m/s", "kW")
    if(!is.null(cp)) {
      names <- append(names, "cp")
      attr(curvpot, "units") <- append(attr(curvpot, "units"), "-")
    }
    if(!is.null(ct)) {
      names <- append(names, "ct")
      attr(curvpot, "units") <- append(attr(curvpot, "units"), "-")
    }
    names(curvpot) <- names
    attr(curvpot, "rho") <- rho
    attr(curvpot, "rated.power") <- rated.p
    if(!is.null(desc)) attr(curvpot, "description") <- desc
    attr(curvpot, "call") <- list(func="curvpot.default")

    class(curvpot) <- "curvpot"
    return(curvpot)
  }

curvpot.leer <-
  function(curvpot, ...) {
    ### Importando curva de potencia de archivo wASP .wgt o archivo de WindPower .pow

    if(system.file(package="ENFICC", "curvasPotencia", curvpot)!="") curvpot <- system.file(package="ENFICC", "curvasPotencia", curvpot)
    if(!file.exists(curvpot)) stop("Archivo no encontrado")

    type <- substr(curvpot, nchar(curvpot)-3, nchar(curvpot))
    if(!any(c(".pow", ".wtg")==type)) stop("No se puede operar el archivo - solo archivos .wtg (WAsP) y .pow (WindPower) son aceptados")

    r <- NULL
    if(type==".pow") {
      pow <- read.table(curvpot, as.is=TRUE)
      cut.out <- as.numeric(pow[4,1])
      if(is.na(cut.out) || is.null(cut.out)) stop("Archivo no encontrado")
      v <- seq(1, cut.out, 1)
      p <- tail(pow, -1)
      suppressWarnings(if(is.na(as.numeric(tail(p, 1)))) p <- head(p, -1))
      p <- as.numeric(p[5:(cut.out+4),1])
      desc <- pow[1,1]
      r <- curvpot.default(curvpot=list(v=v, p=p), rho=1.225, desc=desc)
      attr(r, "call") <- list(func="curvpot.read", curvpot=curvpot)
    } else if(type==".wtg") {
      if(!requireNamespace("XML", quietly=TRUE)) stop("Paquete 'XML' requerido para importar archivo WAsP .wtg")
      wtg <- XML::xmlTreeParse(curvpot, asTree=TRUE)
      if(is.null(wtg$doc$children$WindTurbineGenerator)) stop("Archivo no encontrado")
      n <- length(wtg$doc$children$WindTurbineGenerator)
      idx <- 3
      if(n>4) {
        rho <- NULL
        for(i in 3:(n-1)) rho <- append(rho, as.numeric(XML::xmlAttrs(wtg$doc$children$WindTurbineGenerator[[i]])[["AirDensity"]]))
        idx <- which.min(abs(rho-1.225))+2
      }
      rho <- as.numeric(XML::xmlAttrs(wtg$doc$children$WindTurbineGenerator[[idx]])[["AirDensity"]])
      n <- length(wtg$doc$children$WindTurbineGenerator[[idx]][["DataTable"]])
      v <- p <- ct <- NULL
      for(i in 1:n) {
        v <- append(v, as.numeric(XML::xmlAttrs(wtg$doc$children$WindTurbineGenerator[[idx]][["DataTable"]][[i]])[["WindSpeed"]]))
        p <- append(p, as.numeric(XML::xmlAttrs(wtg$doc$children$WindTurbineGenerator[[idx]][["DataTable"]][[i]])[["PowerOutput"]])/1000)
        ct <- append(ct, as.numeric(XML::xmlAttrs(wtg$doc$children$WindTurbineGenerator[[idx]][["DataTable"]][[i]])[["ThrustCoEfficient"]]))
      }
      desc <- XML::xmlAttrs(XML::xmlRoot(wtg))[["Description"]]
      r <- curvpot.default(curvpot=list(v=v, p=p, ct=ct), rho=rho, desc=desc)
      attr(r, "call") <- list(func="curvpot.read", curvpot=curvpot)
    }

    class(r) <- "curvpot"
    return(r)
  }
