#' @title Grafica una rosa de los vientos
#' @name rosaViento
#' @description  Grafica una rosa de los vientos a partir de los datos recibidos.
#'
#' @export
#' @importFrom grDevices colorRampPalette
#' @importFrom RColorBrewer brewer.pal
#' @importFrom ggplot2 ggplot aes geom_bar scale_x_discrete waiver
#'  coord_polar scale_fill_manual theme element_blank ylim
#'
#' @param data A data.frame containing the wind information (data.frame)
#' @param spd The column of the wind speeds in the "data"-data.frame
#' (numeric)
#' @param dir The column of the wind directions in the "data"-data.frame
#' (numeric)
#' @param spdres The increment of the wind speed legend. Default is 2
#' (numeric)
#' @param dirres The size of the wind sectors. Default is 10  (numeric)
#' @param spdmin Minimum wind speed. Default is 1 (numeric)
#' @param spdmax Maximal wind speed. Default is 30 (numeric)
#' @param palette A color palette used for drawing the wind rose (character)
#' @param debug For running a debug. Default is 0 (numeric)
#' @param spdseq A wind speed sequence, that is used for plotting (numeric)
#' @param countmax g
#'
#' @return NULL
#' 
#' @examples g
#' @author Carlos Ramirez
rosaViento <- function(data, spd, dir, spdres = 2, dirres = 30, spdmin = 2, spdmax = 20,
                          spdseq = NULL, palette = "YlGnBu", countmax = NA, debug = 0){

    # Look to see what data was passed in to the function
  if (is.numeric(spd) & is.numeric(dir)){
    # assume that we've been given vectors of the speed and direction vectors
    data <- data.frame(spd = spd,
                       dir = dir)
    spd = "spd"
    dir = "dir"
  } else if (exists("data")){
    # Assume that we've been given a data frame, and the name of the speed
    # and direction columns. This is the format we want for later use.
  }

  # Tidy up input data ----
  n.in <- NROW(data)
  dnu <- (is.na(data[[spd]]) | is.na(data[[dir]]))
  data[[spd]][dnu] <- NA
  data[[dir]][dnu] <- NA

  # figure out the wind speed bins ----
  if (missing(spdseq)){
    spdseq <- seq(spdmin,spdmax,spdres)
  } else {
    if (debug >0){
      cat("Usando bins de velocidad por defecto \n")
    }
  }
  # get some information about the number of bins, etc.
  n.spd.seq <- length(spdseq)
  n.colors.in.range <- n.spd.seq - 1

  # create the color map
  spd.colors <- colorRampPalette(brewer.pal(min(max(3,n.colors.in.range), min(9,n.colors.in.range)),
                                            palette))(n.colors.in.range)

  if (max(data[[spd]],na.rm = TRUE) > spdmax){
    spd.breaks <- c(spdseq,
                    max(data[[spd]],na.rm = TRUE))
    spd.labels <- c(paste(c(spdseq[1:n.spd.seq-1]), '-',c(spdseq[2:n.spd.seq])),
                    paste(spdmax,"-",max(data[[spd]],na.rm = TRUE)))
    spd.colors <- c(spd.colors, "grey50")
  } else{
    spd.breaks <- spdseq
    spd.labels <- paste(c(spdseq[1:n.spd.seq-1]),
                        '-',
                        c(spdseq[2:n.spd.seq]))
  }
  data$spd.binned <- cut(x = data[[spd]],
                         breaks = spd.breaks,
                         labels = spd.labels,
                         ordered_result = TRUE)
  # clean up the data
  data. <- na.omit(data)

  # figure out the wind direction bins
  dir.breaks <- c(-dirres/2,
                  seq(dirres/2, 360-dirres/2, by = dirres),
                  360+dirres/2)
  dir.labels <- c(paste(360-dirres/2,"-",dirres/2),
                  paste(seq(dirres/2, 360-3*dirres/2, by = dirres),
                        "-",
                        seq(3*dirres/2, 360-dirres/2, by = dirres)),
                  paste(360-dirres/2,"-",dirres/2))
  # assign each wind direction to a bin
  dir.binned <- cut(data[[dir]],
                    breaks = dir.breaks,
                    ordered_result = TRUE)
  levels(dir.binned) <- dir.labels
  data$dir.binned <- dir.binned

  # Run debug if required ----
  if (debug>0){
    cat(dir.breaks,"\n")
    cat(dir.labels,"\n")
    cat(levels(dir.binned),"\n")
  }

  # deal with change in ordering introduced somewhere around version 2.2
  if(packageVersion("ggplot2") > "2.2"){
    data$spd.binned = with(data, factor(spd.binned, levels = rev(levels(spd.binned))))
    spd.colors = rev(spd.colors)
  }

  # create the plot ----
  labels <- FALSE
  if(dirres==90) r.names <- c("N","E","S","W"); labels <- TRUE
  if(dirres==45) r.names <- c("N","NE","E","SE","S","SW","W","NW"); labels <- TRUE
  if(dirres==30) r.names <- c("N","NNE","ENE","E","ESE","SSE","S","SSW","WSW","W","WNW","NNW"); labels <- TRUE
  if(dirres==22.5) r.names <- c("N","NNE","NE","ENE","E","ESE","SE","SSE","S","SSW","SW","WSW","W","WNW","NW","NNW"); labels <- TRUE

  if(labels==TRUE) {
    p.windrose <- ggplot(data = data,
                         aes(x = dir.binned,
                             fill = spd.binned
                             ,y = (..count..)/sum(..count..)
                         ))+
      geom_bar() +
      scale_x_discrete(drop = FALSE,
                       labels = r.names) +
      coord_polar(start = -((dirres/2)/360) * 2*pi) +
      scale_fill_manual(name = "Velocidad de viento [m/s]",
                        values = spd.colors,
                        drop = FALSE) +
      scale_y_continuous(labels = percent) +
      ylab("Frecuencia") + 
      theme_grey() + theme(axis.title.x = element_blank()) + theme(text = element_text(size=18))
  } else {
    p.windrose <- ggplot(data = data,
                         aes(x = dir.binned,
                             fill = spd.binned
                             ,y = (..count..)/sum(..count..)
                         ))+
      geom_bar() +
      scale_x_discrete(drop = FALSE,
                       labels = waiver()) +
      coord_polar(start = -((dirres/2)/360) * 2*pi) +
      scale_fill_manual(name = "Velocidad de viento [m/s]",
                        values = spd.colors,
                        drop = FALSE) +
      scale_y_continuous(labels = percent) +
      ylab("Frecuencia") + 
      theme_grey() + theme(axis.title.x = element_blank()) + theme(text = element_text(size=18))
  }


  # adjust axes if required
  if (!is.na(countmax)){
    p.windrose <- p.windrose +
      ylim(c(0,countmax))
  }

  # print the plot
  print(p.windrose)

  # return the handle to the wind rose
  return(p.windrose)
}
