#' Creacion de series de datos
#'
#' @param altura Altura de medición en metros m como valor numérico
#' @param desc Información en texto plano sobre la serie, señales, instrumentos,
#' entre otros, en formato "string" (opcional).
#' @param v.prom Promedio de la velocidad horizontal del viento en un intervalo en m/s
#' como vector numérico (opcional si se da al menos otra señal).
#' @param v.max Velocidad horizontal máxima del viento en un intervalo en m/s
#' como vector numérico (opcional si se da al menos otra señal).
#' @param v.min Velocidad horizontal mínima del viento en un intervalo en m/s
#' como vector numérico (opcional si se da al menos otra señal).
#' @param v.std Desviación estándar de la velocidad horizontal del viento en un intervalo en m/s
#' como vector numérico (opcional si se da al menos otra señal).
#' @param dir.prom Promedio de la dirección del viento en un intervalo en grados
#' como vector numérico (opcional si se da al menos otra señal).
#' @param dir.std Desviación estándar de la dirección del viento en un intervalo en grados
#' como vector numérico (opcional si se da al menos otra señal).
#' @param tmp Temperatura en °C como vector numérico (opcional si se da al menos otra señal).
#' @param press Presión en hPa como vector numérico (opcional si se da al menos otra señal)
#' @param hum Humedad relativa en \% como vector numèrico (opcional si se da al menos otra señal)
#' @param ... Otras señales tales como radiaciòn solar, velocidad vertical, etc. como vector numérico.
#'
#' @description Crea un objeto con las series de datos de todas las señales
#' para una altura de medición.
#'
#' @details Los anemómetros está ubicados en pares, cada par a una altura determinada.
#' Las veletas se ubican entre 4 y 10 metros por debajo de los pares de anemómetros o a las mismas alturas.
#' Se puede combinar la información de ambos casos (velocidad y dirección) en una misma
#' serie de datos con el uso de este método, teniendo la altura de los anemómetros
#' como referencia. La serie de datos debe contener una sola señal de velocidad por altura,
#' lo cual implica que ya se ha incluido la información del anemómetro redundante.
#'
#' Generalmente se manejan intervalos de medición diezminutales, pero otros intervalos
#' son aplicables.
#'
#' Para series de datos que contienen información de la velocidad promedio \code{v.prom} y
#' la desviación estándar \code{v.std}, la intensidad de turbulencia es calculada y
#' sumada a los datos, con el nombre \code{turb.int}.
#'
#' @return El método devuelve un objeto con la siguiente información:
#'
#' \code{altura} Altura de medición.
#'
#' \code{data} Datos con las señales medidas
#'
#' @examples
#' # Inicia Ejemplo
#' @export

serie <- function (altura, desc, v.prom, v.max, v.min, v.std, dir.prom,
          dir.std, tmp, press, hum, ...)
{
  #Verificación de parámetros de entrada
  if (missing(altura))
    stop("'altura' es obligatoria")
  if (!is.numeric(altura))
    stop("'altura' debe ser un valor numérico")
  if (missing(v.prom) && missing(v.max) && missing(v.min) &&
      missing(v.std) && missing(dir.prom) && missing(dir.std) &&
      missing(tmp) && missing(press) && missing(hum) &&
      missing(...))
    stop("No hay datos")
  if (missing(desc))
    desc <- NULL
  if (missing(v.prom))
    v.prom <- NULL
  if (missing(v.max))
    v.max <- NULL
  if (missing(v.min))
    v.min <- NULL
  if (missing(v.std))
    v.std <- NULL
  if (missing(dir.prom))
    dir.prom <- NULL
  if (missing(dir.std))
    dir.std <- NULL
  if (missing(tmp))
    tmp <- NULL
  if (missing(press))
    press <- NULL
  if (missing(hum))
    hum <- NULL
  #Cálculo de intensidad de turbulencia si existe v.prom y v.std
  if (!is.null(v.std) && !is.null(v.prom))
    turb.int <- v.std/v.prom
  else turb.int <- NULL
  #Crea la serie de datos "data" y asigna unidades a las variables
  data <- data.frame(cbind(v.prom, v.max, v.min, v.std, dir.prom,
                           dir.std, turb.int, tmp, press, hum, ...))
  attr(altura, "unit") <- "m"
  if (!is.null(data$v.prom))
    attr(data$v.prom, "unit") <- "m/s"
  if (!is.null(data$v.max))
    attr(data$v.max, "unit") <- "m/s"
  if (!is.null(data$v.min))
    attr(data$v.min, "unit") <- "m/s"
  if (!is.null(data$v.std))
    attr(data$v.std, "unit") <- "m/s"
  if (!is.null(data$dir.prom))
    attr(data$dir.prom, "unit") <- "/°"
  if (!is.null(data$dir.std))
    attr(data$dir.std, "unit") <- "/°"
  if (!is.null(data$tmp))
    attr(data$tmp, "unit") <- "°C"
  if (!is.null(data$press))
    attr(data$press, "unit") <- "hPa"
  if (!is.null(data$hum))
    attr(data$hum, "unit") <- "/%"
  if (!is.null(data$turb.int))
    attr(data$turb.int, "unit") <- "-"
  if (is.null(desc))
    l <- list(altura = altura, data = data)
  else l <- list(altura = altura, description = desc, data = data)
  class(l) <- "serie"
  return(l)
}
