#' Creacion de torre meteorologica
#' 
#' @description  Crea un objeto que contiene una o más series de datos (alturas de medición) por torre meteorológica.
#' Todas las series de datos son ordenadas según la altura en orden ascendente.
#'
#' @param marcatemporal Marca temporal como un vector POSIXlt creado por [marcatemporal]
#' @param ... Al menos una serie de datos creada por [serie].
#' @param pos Coordenadas de longitud y latitud del sitio en grados decimales como vector de dos valores numéricos
#' @param radioRep Radio de representatividad de la torre de medición
#' @param desc Información en texto plano sobre el nombre de la torre, mediciones, condición de la torre meteorológica, entre otros, como string
#'
#' @details Información útil sobre un torre meteorológica generalmente es obtenida del protocolo de instalación.
#'
#' Mediciones de velocidad de viento a diversas alturas son requeridas para el cálculo del perfil de cortante del viento del sitio.
#'
#' @return El método entrega un objeto con la torre meteorológica, la cual es necesaria para los análisis de datos y el cálculo final del ENFICC.
#' Este objeto contiene la siguiente información:
#'
#' \code{marcatemporal} Marca temporal de las observaciones
#'
#' \code{posicion} Coordenadas en latitud y longitud (opcional)
#'
#' \code{descripcion} Información de la torre (opcional)
#'
#' \code{series} Lista de una o más series de datos que consisten de la altura de medición y los respectivos datos.
#' 
#' @export
#'
#' @examples ## Empieza ejemplo

torre <- function (marcatemporal, ..., pos = NULL, radioRep = NULL, desc = NULL)
{
  if (!is.null(pos)) {
    if (!is.numeric(pos))
      stop("Las coordenadas geográficas deben ser ingresadas en grados decimales")
    if (pos[1] > 180 || pos[1] < (-180) || pos[2] > 90 ||
        pos[2] < (-90))
      stop("Las coordenadas están fuera de rango - Por favor usar grados decimales")
  }
  if (!is.null(radioRep)) {
    if (!is.numeric(radioRep))
      stop("El radio de representatividad debe ser un un valor numérico en kilómetros")
  }
  l <- list(...)
  num.series <- length(l)
  if (num.series < 1)
    stop("No hay datos - Por favor agregue otra serie de datos")
  for (i in 1:num.series) if (class(l[[i]]) != "serie")
    stop(names(l)[i], " No es un objeto tipo serie")
  if (any(class(marcatemporal) == "POSIXlt") == FALSE)
    stop("'marcatemporal' debe ser entregado en formato POSIXlt")
  if (length(marcatemporal) != length(l[[1]]$data[, 1]))
    stop("Las series y las marcas temporales son vectores de diferente longitud")
  var.names <- NULL
  for (i in 1:num.series) var.names <- append(var.names, names(l[[i]]$data))
  var.names <- unique(var.names)
  units <- data.frame(matrix(NA, nrow = num.series, ncol = length(var.names) + 1))
  names(units) <- c("altura", var.names)
  for (i in 1:num.series) {
    units[i, 1] <- attr(l[[i]]$altura, "unit")
    for (j in 2:dim(units)[2]) {
      if (!is.null(l[[i]]$data[1, names(units)[j]]))
        if (!is.null(attr(l[[i]]$data[, names(units)[j]],
                          "unit")))
          units[i, j] <- attr(l[[i]]$data[, names(units)[j]],
                              "unit")
    }
  }
  for (i in 1:dim(units)[2]) {
    if (length(unique(units[, i])[!is.na(unique(units[, i]))]) >
        1)
      stop("Different units in ", names(units)[i], " - ",
           paste(unique(units[, i])[!is.na(unique(units[,
                                                        i]))], collapse = ", "))
  }
  if (length(l) > 1) {
    alturas <- c()
    for (i in 1:length(l)) alturas <- append(alturas, l[[i]]$altura)
    altura.idx <- cbind(alturas, seq(1:length(alturas)))
    l <- l[altura.idx[order(altura.idx[, 1]),
                      ][, 2]]
  }
  if (is.null(names(l)))
    for (i in 1:length(l)) names(l)[i] <- paste0("serie", i)
  else for (i in 1:length(l)) if (names(l)[i] == "")
    names(l)[i] <- paste0("serie", i)
  
  r <- list(marcatemporal = marcatemporal, posicion = pos,
            RadioRep = radioRep, descripcion = desc, series = l)
  class(r) <- "torre"
  return(r)
}
