#' Calculo de Velocidades Afectadas por Estela para cada Turbina
#'
#' @param reconstruccion R
#' @param torres_MET T
#' @param serie_altura_cubo S
#' @param rho10min R
#' @param parqueEolico P
#' @param curvascorregidas C
#' @param diametro D
#' @param altCubo A
#' @param onshore True si es onshore, False si es offshore
#'
#' @return Serie de Velocidades Afectadas por Estela para cada Turbina
#' @export
#'
#' @examples #Empieza ejemplo

velocidadEstela <- function(reconstruccion, torres_MET, serie_altura_cubo, rho10min, parqueEolico, 
                            curvascorregidas, diametro, altCubo, onshore)
{
  #Se verifica la entrada de los parámetros
  if (missing(serie_altura_cubo))
    stop("No hay datos de velocidad horizontal a la altura del cubo.")
  if (missing(parqueEolico))
    stop("No hay datos de coordenadas de las turbinas en el parque eólico.")
  if (missing(reconstruccion))
    stop("Se requieren las reconstrucciones históricas de dirección.")
  if (missing(rho10min))
    stop("Se debe ingresar la serie de densidad en sitio para calcular cambios horarios en la curva de empuje")
  #Radio de la Turbina
  radtur <- diametro/2
  arearot <- pi*(radtur^2)
  #Se importan las fechas de la reconstrucción
  fechaRECON <- strptime(reconstruccion[[1]][,2], "%Y-%m-%d %H:%M:%S", "")
  
  numTorres <- length(torres_MET)
  #Se calcula el día típico por mes de la densidad en sitio
  densidad_meses <- lapply(1:numTorres, function(n) {
    r <- as.POSIXct(torres_MET[[n]]$marcatemporal)
    rhom <- data_frame(fecha=r,rho=rho10min[[n]])
    rhomeses <- split(rhom, format(rhom$fecha, "%m"))
    for (i in 1:length(rhomeses)) {
      df <- as.data.frame(matrix(ncol = 2,nrow = length(rhomeses[[i]]$rho)))
      df[,1] <- rhomeses[[i]]$fecha
      df[,2] <- rhomeses[[i]]$rho
      rhomeses[[i]] <- aggregate(V2~format(V1,"%H"), data=df, FUN=mean)
    }
    return(rhomeses)
  })

  #Se importa la curva de coeficiente de empuje (cth) y se crea la función de ct vs. velocidad
  curvaCt <- lapply(1:numTorres, function(n) {
    curva_vel <- curvascorregidas[[n]]$curvaCT$v
    curva_cth <- curvascorregidas[[n]]$curvaCT$ct
    return(approxfun(curva_vel, curva_cth))
  })
  #Número de Turbinas
  NT <- length(parqueEolico[[1]])
  #Se refieren las posiciones de las turbinas con la primera de la lista para generar ubicaciones en metros
  parqueEnMetros <- parqueEolico
  x_m <- rep(0,NT)
  y_m <- rep(0,NT)
  parqueEnMetros <- cbind(parqueEnMetros, x_m, y_m)
  for(i in 2:NT){
    longT1 <- parqueEnMetros[[2]][1]
    longT2 <- parqueEnMetros[[2]][i]
    latT1 <- parqueEnMetros[[3]][1]
    latT2 <- parqueEnMetros[[3]][i]
    sign_x <- sign(longT2 - longT1)
    sign_y <- sign(latT2 - latT1)
    ##El método distVincentyEllipsoid calcula la distancia basado en el elipsoide WGS84 como modelo del planeta Tierra
    vecturb <- c((sign_x*distVincentyEllipsoid(c(longT2,latT1),c(longT1,latT1))),
                 (sign_y*distVincentyEllipsoid(c(longT1,latT2),c(longT1,latT1))))
    parqueEnMetros[[5]][i] <- vecturb[1]
    parqueEnMetros[[6]][i] <- vecturb[2]
  }
  
  #Se asigna la información de cada torre a cada turbina según ubicación y radio de representatividad
  longTorre <- rep(0,numTorres)
  latTorre <- rep(0,numTorres)
  radioTorre <- rep(0,numTorres)
  indiceTorre <- rep(0,numTorres)
  for (i in 1:numTorres) {
    longTorre[i] <- torres_MET[[i]]$posicion[1]
    latTorre[i] <- torres_MET[[i]]$posicion[2]
    radioTorre[i] <- torres_MET[[i]]$RadioRep
    indiceTorre[i] <- i
  }
  posTorres <- data.frame(Longitud=longTorre, Latitud=latTorre, RadioRep=radioTorre, indiceTorre=indiceTorre)
  x_m <- rep(0,numTorres)
  y_m <- rep(0,numTorres)
  posTorres <- cbind(posTorres, x_m, y_m)
  
  for(i in 1:numTorres) {
    longTurb <- parqueEnMetros[[2]][1]
    longTorre <- posTorres$Longitud[i]
    latTurb <- parqueEnMetros[[3]][1]
    latTorre <- posTorres$Latitud[i]
    sign_x <- sign(longTorre - longTurb)
    sign_y <- sign(latTorre - latTurb)
    ##El método distVincentyEllipsoid calcula la distancia basado en el elipsoide WGS84 como modelo del planeta Tierra
    vecturb <- c((sign_x*distVincentyEllipsoid(c(longTorre,latTurb),c(longTurb,latTurb))),
                 (sign_y*distVincentyEllipsoid(c(longTurb,latTorre),c(longTurb,latTurb))))
    posTorres[[5]][i] <- vecturb[1]
    posTorres[[6]][i] <- vecturb[2]
  }
  
  indice_Torre <- rep(0,NT)
  parqueEnMetros <- cbind(parqueEnMetros,indice_Torre)
  for(i in 1:NT) {
    distancia <- rep(0,numTorres)
    x1 <- parqueEnMetros$x_m[i]
    y1 <- parqueEnMetros$y_m[i]
    for(j in 1:numTorres) {
      x2 <- posTorres$x_m[j]
      y2 <- posTorres$y_m[j]
      distancia[j] <- sqrt((x2 - x1)^2 + (y2 - y1)^2)/1000
    }
    indice <- which.min(distancia)
    if(distancia[indice] < posTorres$RadioRep[indice]) {
      parqueEnMetros$indice_Torre[i] <- indice
    } else {
      stop(paste0("La turbina ", i, " se encuentra por fuera del radio de representatividad de las torres de medición en sitio."))
    }
  }

  withProgress(message = 'Calculando las velocidades incidentes afectadas por estela. Espere por favor.', value = 0, {
  #Se ejecuta el algoritmo con la estela cambiando por dirección en cada paso de tiempo
  cores <- detectCores()
  registerDoParallel(makeCluster(cores))
  velocidades_estela <- foreach(i=1:length(reconstruccion[[1]][,3])) %dopar% {
    #Se leen las direcciones de todas las torres para el paso de tiempo
    serie_dir <- rep(0,numTorres)
    for (j in 1:numTorres) {
      serie_dir[j] <- reconstruccion[[j]][,4][i]
    }
    
    #Se reorganiza el parque eólico segùn la dirección del viento
    theta_prom <- mean(serie_dir)
    a <- sin(theta_prom*pi/180)
    b <- cos(theta_prom*pi/180)
    assign("IndiceT", parqueEnMetros[[1]])
    TurbReorg <- data.frame(cbind(IndiceT))
    TurbReorg["Reubicacion"] <- a*parqueEnMetros[[5]] + b*parqueEnMetros[[6]]
    TurbReorg <- TurbReorg[with(TurbReorg,order(-Reubicacion)),]
    parqueReorg <- parqueEnMetros
    borrar <- c("Longitud","Latitud")
    parqueReorg <- parqueReorg[ , !(names(parqueReorg) %in% borrar)]
    for (j in 1:NT){
      if(!parqueEolico[[1]][j]==TurbReorg$IndiceT[j]) {
        parqueReorg[[1]][j] <- parqueEnMetros[[1]][TurbReorg$IndiceT[j]]
        parqueReorg[[2]][j] <- parqueEnMetros[[4]][TurbReorg$IndiceT[j]]
        parqueReorg[[3]][j] <- parqueEnMetros[[5]][TurbReorg$IndiceT[j]]
        parqueReorg[[4]][j] <- parqueEnMetros[[6]][TurbReorg$IndiceT[j]]
        parqueReorg[[5]][j] <- parqueEnMetros[[7]][TurbReorg$IndiceT[j]]
      }
    }

    #Conversión de dirección del viento a radianes con referencia convencional
    dir <- (90-serie_dir)*(pi/180)
    #Direcciones de cada turbina
    dirturb <- rep(0,NT)
    for(j in 1:NT) {
      dirturb[j] <- dir[parqueReorg$indice_Torre[j]]
    }
    #Velocidades incidentes de las turbinas sin afectar por la estela según la torre
    voo <- rep(0,NT)
    for(j in 1:NT) {
      voo[j] <- serie_altura_cubo[[parqueReorg$indice_Torre[j]]][i]
    }
    #Se inicializan las velocidades de las turbinas en el paso de tiempo con las velocidades sin afectar
    velturb <- voo

    for(T1 in 1:NT){
      #Reinicialización de variables
      suma <- rep(0,NT)
      ashad <- rep(0,NT)
      vwx <- rep(0,NT)
      #Cálculo del efecto de estela de cada turbina T2 sobre la turbina T1
      for(T2 in 1:NT){
        if (T1!=T2){
          #Vector unitario de dirección del viento
          vecdir <- c(-cos(dirturb[T2]),-sin(dirturb[T2]),0)
          #Cálculo del vector entre las turbinas T1 Y T2
          vecturb <- c(parqueReorg[[3]][T1]-parqueReorg[[3]][T2],parqueReorg[[4]][T1]-parqueReorg[[4]][T2],
                       (parqueReorg[[2]][T1]+altCubo)-(parqueReorg[[2]][T2]+altCubo))
          #Cálculo de la proyección (producto punto) del vector entre las turbinas T1 y T2 sobre el vector de dirección del viento
          proy <- vecturb%*%vecdir
          if (proy > 0) {
            #Cálculo del centro de la estela de T2 a la altura de T1
            centro <- c((parqueReorg[[3]][T2]+proy*vecdir[1]),
                        (parqueReorg[[4]][T2]+proy*vecdir[2]),(parqueReorg[[2]][T2]+altCubo))
            #Cálculo del radio de la estela T2 a la altura de T1
            if(onshore==TRUE) radio <- abs(radtur+(0.075*proy))
            else radio <- abs(radtur+(0.04*proy))
            #Cálculo del coeficiente de empuje instantáneo
            mRHO <- format(fechaRECON[i], "%m")
            hRHO <- as.numeric(format(fechaRECON[i], "%H"))+1
            C_th <- curvaCt[[parqueReorg$indice_Torre[T2]]](velturb[T2])*
              (mean(rho10min[[parqueReorg$indice_Torre[T2]]])/densidad_meses[[parqueReorg$indice_Torre[T2]]][[mRHO]][,2][hRHO])
            #Cálculo de la velocidad de la estela de T2 sobre la turbina T1
            vwx <- velturb[T2]*(1-(1-sqrt(1-C_th))*(radtur/radio)^(2))
            #Distancia entre el centro de la estela T2 y el centro del rotor de la turbina T1
            d <- abs(norm(c(parqueReorg$x_m[T1]-centro[1],parqueReorg$y_m[T1]-centro[2],
                            parqueReorg[[2]][T1]+altCubo-centro[3]), type="2"))
            #Cálculo del área de efecto de la estela de T2 sobre el rotor de la turbina T1
            if (d >= (radtur+radio)) {
              ashad[T2] <- 0
            } else if (d >= radio) {
              d1 <- abs(((radtur^2)-(radio^2)+(d^2))/(2*d))
              z <- abs(((radtur^2)-(d1^2))^0.5)
              ashad[T2] <- (radtur^2)*acos(d1/radtur)+(radio^2)*acos((d-d1)/radio)-(d*z)
            } else if ((d+radtur) > radio && d > ((radio^2-radtur^2)^0.5)) {
              d1 <- abs(((radtur^2)-(radio^2)+(d^2))/(2*d))
              z <- abs(((radtur^2)-(d1^2))^0.5)
              ashad[T2] <- (radtur^2)*acos(d1/radtur)+(radio^2)*acos((d-d1)/radio)-(d*z)
            } else if ((d+radtur) > radio && d < ((radio^2-radtur^2)^0.5)) {
              d1 <- abs(((radtur^2)-(radio^2)+(d^2))/(2*d))
              z <- abs(((radtur^2)-(d1^2))^0.5)
              ashad[T2] <- arearot-(radtur^2)*acos(d1/radtur)+(radio^2)*acos((d+d1)/radio)-(d*z)
            } else {
              ashad[T2]=arearot
            }
            #Cálculo de la sumatoria para la estimación de la velocidad incedente en el rotor de cada turbina
            #teniendo en cuenta el efecto estela
            suma[T1]=suma[T1]+((ashad[T2]/arearot)*(vwx-voo[T1])^2)
          }
        }
      }
      velturb[T1]=voo[T1]-(suma[T1])^0.5 
      if(is.na(velturb[T1])) {
        velturb[T1]<-voo[T1]
      }
      if(velturb[T1]<0) {
        velturb[T1]<-0
      }
    }
    velturb_final <- rep(0,NT)
    for (j in 1:NT){
      if(!parqueEolico[[1]][j]==parqueReorg[[1]][j]) {
        devuelta <- which(parqueReorg[[1]]==parqueEolico[[1]][j])
        velturb_final[j] = velturb[devuelta]
      } else {
        velturb_final[j] = velturb[j]
      }
    }
    return(velturb_final)
  }
  })
  
  #Se crea un objeto con todas las series de datos de velocidad por turbina,
  #inicializada con la serie de velocidades no afectada por estela
  VLibres <- data.frame(serie_altura_cubo[[parqueEnMetros$indice_Torre[1]]])
  assign("Turbina_1", rep(0,length(reconstruccion[[1]][,3])))
  VelFinal <- data.frame(cbind(Turbina_1))
  for (i in 2:NT){
    VelFinal[paste("Turbina_", i, sep = "")] <- rep(0,length(reconstruccion[[1]][,3]))
    VLibres <- cbind(VLibres,serie_altura_cubo[[parqueEnMetros$indice_Torre[i]]])
  }
  VelLibreProm <- rowMeans(VLibres, na.rm=TRUE)
  #Se asignan los valores obtenidos en la lista 'velocidades_estela' al data.frame 'VelFinal'
  withProgress(message = 'Reorganizando la informacion calculada. Espere por favor.', value = 0, {
  for (i in 1:NT) {
    incProgress(1/NT)
    for (j in 1:length(reconstruccion[[1]][,3])) {
      VelFinal[[i]][j] <- velocidades_estela[[j]][i]
    }
  }
  })
  VelFinal <- cbind(Fecha=as.POSIXct(fechaRECON), Velocidad_Libre_Parque=VelLibreProm, VelFinal)
  
  return(VelFinal)
  stopCluster(makeCluster(cores))
}
